<?php

/**
 * Plugin Name: AutoBlogGPT: AI-Powered Blog Creator
 * Description: Automatically generate high-quality blog posts using ChatGPT. This plugin creates custom blog posts and can attach featured images from URLs. When a blog post is made public, the plugin will request AI-generated content from the ChatGPT API. Includes templates and shortcode for display.
 * Version: 1.1.0
 * Author: Codemenschen
 * License: GPLv2 or later
 */

if (! defined('ABSPATH')) {
    exit;
}


// Register the REST API route for creating blog posts
add_action('rest_api_init', function () {
    register_rest_route('autobloggpt/v1', '/autobloggpt-service', array(
        'methods' => 'POST',
        'callback' => 'autobloggpt_service_function',
        'permission_callback' => '__return_true',  // No permission check
    ));

    register_rest_route('autobloggpt/v1', '/save-code', array(
        'methods' => 'POST',
        'callback' => 'save_autobloggpt_security_code',
        'permission_callback' => function () {
            return true;
        }
    ));
});
include_once 'autobloggpt_function.php';
include_once 'cmsbuddy_client_function.php';
function autobloggpt_service_function(WP_REST_Request $request)
{
    ob_clean();
    // Extract the action parameter to determine create, edit, delete, etc.
    $action = sanitize_text_field($request->get_param('action'));
    $post_type = sanitize_text_field($request->get_param('post_type'));

    // Call the appropriate function based on the action parameter
    switch ($action) {
        case 'get-post':
            return autobloggpt_get_post($request);

        case 'create-post-by-post-type':
            return autobloggpt_create_post_by_post_type($request);

        case 'create-post':
            if ($post_type == 'tribe_events') {
                return autobloggpt_create_event($request);
            } else {
                return autobloggpt_create_post($request);
            }
        case 'edit-post':
            if ($post_type == 'tribe_events') {
                return autobloggpt_edit_event($request);
            } else {
                return autobloggpt_edit_post($request);
            }
        case 'delete-post':
            if ($post_type == 'tribe_events') {
                return autobloggpt_delete_event($request);
            } else {
                return autobloggpt_delete_post($request);
            }

        case 'list-posts':
            if ($post_type == 'tribe_events') {
                return autobloggpt_list_events($request);
            } else {
                return autobloggpt_list_posts($request);
            }
        case 'preview-post':
            return autobloggpt_preview_post($request);
        case 'fetch_html_source':
            return fetch_html_source($request);
        case 'fetch_wp_site_data': // New action
            return fetch_wp_site_data($request); // Call the new function
        case 'get_post_data_by_url': // New action
            return get_post_data_by_url($request); // Call the new function
        case 'get_all_post_types': // New action
            return get_all_post_types_from_db(); // Call the new function
        default:
            return new WP_Error('invalid_action', 'Invalid action', array('status' => 400));
    }
}

/**
 * Shortcode to display blog posts
 */
function autobloggpt_blog_shortcode($atts)
{
    // Get the tags from the plugin settings or shortcode attributes
    $tags = esc_attr(get_option('autobloggpt_tags')); // Get the saved tags option

    // Check if tags are provided, split them into an array
    $tag_array = ! empty($tags) ? explode(',', $tags) : array();

    // Query for blog posts based on tags
    $query_args = array(
        'post_type' => 'post',
        'posts_per_page' => 5,  // Adjust as needed
    );

    if (! empty($tag_array)) {
        $query_args['tag'] = $tag_array; // Add tag filter to the query
    }

    $query = new WP_Query($query_args);

    if ($query->have_posts()) {
        $output = '<div class="autobloggpt-blogs">';
        while ($query->have_posts()) :
            $query->the_post();
            $output .= '<h2>' . get_the_title() . '</h2>';
            $output .= '<div>' . get_the_excerpt() . '</div>';
            if (has_post_thumbnail()) {
                $output .= get_the_post_thumbnail(get_the_ID(), 'thumbnail');
            }
            $output .= '<a href="' . get_permalink() . '">Read more</a>';
        endwhile;
        $output .= '</div>';
    } else {
        $output = 'No blogs found for the selected tags.';
    }

    wp_reset_postdata();
    return $output;
}
add_shortcode('autobloggpt_blogs', 'autobloggpt_blog_shortcode');


// Add a menu item to the WordPress admin dashboard
add_action('admin_menu', 'autobloggpt_add_admin_menu');

function autobloggpt_add_admin_menu()
{
    add_menu_page(
        'AutoBlogGPT',    // Page title
        'AutoBlogGPT',              // Menu title
        'manage_options',           // Capability
        'autobloggpt-dashboard',    // Menu slug
        'autobloggpt_dashboard_page', // Function to display content
        'dashicons-admin-site-alt3', // Icon URL (use a dashicon)
        90                          // Position in the menu
    );
}


// Register settings
function autobloggpt_register_settings()
{
    // Register a setting for each field
    register_setting('autobloggpt_settings_group', 'autobloggpt_license_key'); // License key setting
    register_setting('autobloggpt_settings_group', 'autobloggpt_post_type');
    register_setting('autobloggpt_settings_group', 'autobloggpt_tags');
    register_setting('autobloggpt_settings_group', 'autobloggpt_category');
    register_setting('autobloggpt_settings_group', 'autobloggpt_security_code');
}
add_action('admin_init', 'autobloggpt_register_settings');
// Enqueue the CSS file for the dashboard
function autobloggpt_enqueue_styles()
{
    wp_enqueue_style('autobloggpt-dashboard', plugin_dir_url(__FILE__) . 'assets/css/autobloggpt-dashboard.css');
}
add_action('admin_enqueue_scripts', 'autobloggpt_enqueue_styles');
// Enqueue the CSS file for the dashboard


// Generate or fetch a unique 3-letter security code
function generate_security_code()
{
    // Check if the security code already exists in options
    $security_code = get_option('autobloggpt_security_code');
    if ($security_code) {
        return $security_code; // Return existing code if found
    }

    // Code does not exist, so make an API call to codemenschen.at
    $api_url = 'https://codemenschen.at/wp-json/autobloggpt/v1/generate-code'; // Replace with the actual endpoint

    $response = wp_remote_get($api_url);

    // Check for errors in the API request
    if (is_wp_error($response)) {
        return 'ERR'; // Return a default error code if the API call fails
    }

    // Decode the API response to get the new code
    $body = wp_remote_retrieve_body($response);
    $data = json_decode($body);

    if (!empty($data->code)) {
        $security_code = strtoupper(substr($data->code, 0, 3)); // Ensure code is 3 letters

        // Save the new code to the options for future use
        update_option('autobloggpt_security_code', $security_code);
    } else {
        // If no code is returned, set a default fallback
        $security_code = 'DEF';
    }

    return $security_code;
}


function autobloggpt_dashboard_page()
{

    // Get the current security code or generate it if not existing
    $security_code = get_option('autobloggpt_security_code');


    // Get the current license key from the options table
    $license_key = get_option('autobloggpt_license_key');
    $license_status = ''; // Initialize the status message

    if (! empty($license_key)) {
        $button_text = 'License Activated';
        $license_status = '<div class="notice notice-success" style="background-color: #d4edda; border-color: #c3e6cb; color: #155724; padding: 15px; margin-top: 10px;"><p>Your license is currently activated.</p></div>';
    } else {
        $button_text = 'Activate License';
        $license_status = '<div class="notice notice-warning" style="background-color: #fff3cd; border-color: #ffeeba; color: #856404; padding: 15px; margin-top: 10px;"><p>Please enter your license key to activate AutoBlogGPT.</p></div>';
    }

?>
    <div class="wrap abgpt-dashboard">
        <h1>AutoBlogGPT Dashboard</h1>
        <!-- License Key Form -->
        <div class="abgpt-license-key-section" style="margin-bottom: 20px;">
            <form id="autobloggpt-license-form">
                <div class="abgpt-form-group">
                    <label for="autobloggpt_license_key" style="font-weight: bold; font-size: 14px;">License Key:</label>
                    <input type="password" id="autobloggpt_license_key" name="autobloggpt_license_key"
                        value="<?php echo esc_attr($license_key); ?>"
                        style="width: 100%; padding: 8px; font-size: 16px; border-radius: 4px; border: 1px solid #ccc;" />
                    <span id="toggle-password" style="cursor: pointer; margin-left: 10px;">Show</span>
                </div>
                <input type="submit" id="autobloggpt-submit-btn" class="button button-primary"
                    value="<?php echo $button_text; ?>" style="margin-top: 10px;" />
            </form>

            <!-- Display license status message -->
            <!-- <div id="autobloggpt-message" style="margin-top: 15px;">
                <?php echo $license_status; ?>
            </div> -->
        </div>


        <div class="abgpt-dashboard-container">
            <div class="abgpt-dashboard-container-left">
                <!-- Left side for tabs -->
                <div class="abgpt-dashboard-sidebar">
                    <ul class="abgpt-dashboard-tabs">
                        <li><a href="#tab-1" class="abgpt-tab-link abgpt-nav-tab-active" onclick="loadContent('tab-1')">How
                                to
                                Use</a></li>
                        <li><a href="#tab-2" class="abgpt-tab-link" onclick="loadContent('tab-2')">Settings</a></li>
                    </ul>
                </div>

                <!-- Right side for dynamic content loading -->
                <div class="abgpt-dashboard-content" id="abgpt-dashboard-content-area">
                    <!-- Tab 1: How to Use AutoBlogGPT -->
                    <div id="tab-1" class="abgpt-tab-content" style="display:block;">
                        <section class="abgpt-section-intro">
                            <p>Welcome to the AutoBlogGPT plugin dashboard. Here you can manage and create AI-powered blog
                                posts
                                with ease.</p>
                        </section>
                        <!-- Security Code Section -->
                        <div class="abgpt-security-code-section" style="margin-bottom: 20px;">
                            <h2>Security Code</h2>
                            <p>Your current security code: <strong id="current-security-code"><?php echo esc_html($security_code); ?></strong></p>
                            <button id="renew-security-code" class="button button-secondary">Renew Code</button>
                            <div id="security-code-message" style="margin-top: 10px;"></div>
                        </div>
                        <section class="abgpt-section-how-to-use">
                            <h2>How to Use AutoBlogGPT</h2>
                            <p class="abgpt-use-guide">Use <a href="https://chatgpt.com/g/g-RMXOcI8HQ-autobloggpt" target="_blank">this link</a> to create AI-powered blog posts with AutoBlogGPT. Below is the guide on how to use it:</p>
                            <ol class="abgpt-how-to-list">
                                <li><strong>Step 1:</strong> Provide a topic or idea to ChatGPT, and it will generate a blog post title and content automatically.</li>
                                <li><strong>Step 2:</strong> Once satisfied with the content, type <strong>"publish"</strong> to release it.</li>
                                <li><strong>Step 3:</strong> You’ll be prompted to enter the security code where you want to publish the blog post. Your security code is: <strong><?php echo esc_html(get_option('autobloggpt_security_code')); ?></strong>.</li>
                                <li><strong>Step 4:</strong> ChatGPT will handle creating and publishing the post on your website.</li>
                            </ol>
                        </section>
                        <section class="abgpt-section-shortcode">
                            <h2>Shortcode</h2>
                            <p>Use the following shortcode to display the latest AI-generated blog posts:</p>
                            <pre><code>[autobloggpt_blogs]</code></pre>
                            <p>This shortcode displays the latest AutoBlogGPT-generated posts on your website.</p>
                        </section>
                    </div>

                    <!-- Tab 2: Settings -->
                    <div id="tab-2" class="abgpt-tab-content" style="display:none;">
                        <h2>Settings</h2>
                        <form method="post" action="options.php">
                            <?php
                            settings_fields('autobloggpt_settings_group'); // Register settings group
                            do_settings_sections('autobloggpt_settings_group'); // Output settings fields
                            ?>

                            <!-- Post Type Selection -->
                            <div class="abgpt-form-group">
                                <label for="autobloggpt_post_type">Select Post Type:</label>
                                <select id="autobloggpt_post_type" name="autobloggpt_post_type">
                                    <?php
                                    $post_types = get_post_types(array('public' => true), 'objects');
                                    $selected_post_type = get_option('autobloggpt_post_type', 'post'); // Default to 'post'
                                    foreach ($post_types as $post_type) {
                                        echo '<option value="' . $post_type->name . '" ' . selected($post_type->name, $selected_post_type, false) . '>' . $post_type->labels->singular_name . '</option>';
                                    }
                                    ?>
                                </select>
                            </div>

                            <!-- Tags Input -->
                            <div class="abgpt-form-group">
                                <label for="autobloggpt_tags">Enter Tags (comma-separated):</label>
                                <input type="text" id="autobloggpt_tags" name="autobloggpt_tags"
                                    value="<?php echo esc_attr(get_option('autobloggpt_tags')); ?>" />
                            </div>

                            <!-- Categories Selection -->
                            <div class="abgpt-form-group">
                                <label for="autobloggpt_category">Select Category:</label>
                                <select id="autobloggpt_category" name="autobloggpt_category">
                                    <?php
                                    $categories = get_categories(array(
                                        'hide_empty' => false, // Ensure categories with no posts are also displayed
                                    ));

                                    $selected_category = get_option('autobloggpt_category'); // Get selected option from settings

                                    foreach ($categories as $category) {
                                        echo '<option value="' . $category->term_id . '" ' . selected($category->term_id, $selected_category, false) . '>' . $category->name . '</option>';
                                    }
                                    ?>
                                </select>
                            </div>

                            <?php submit_button(); ?>
                        </form>
                    </div>
                </div>
            </div>
            <div class="abgpt-dashboard-container-right">
                <?php
                include_once(plugin_dir_path(__FILE__) . 'includes/admin_sidebar.php');

                ?>
            </div>

        </div>
    </div>
    <script type="text/javascript">
        jQuery(document).ready(function($) {
            // Handle form submission via AJAX
            $('#autobloggpt-license-form').on('submit', function(e) {
                e.preventDefault(); // Prevent normal form submission

                var licenseKey = $('#autobloggpt_license_key').val(); // Get license key

                // Clear any previous messages
                $('#autobloggpt-message').html('');

                // Send AJAX request to activate the license
                $.ajax({
                    url: '<?php echo admin_url('admin-ajax.php'); ?>', // WordPress AJAX URL
                    type: 'POST',
                    data: {
                        action: 'activate_autobloggpt_license', // AJAX action
                        security: '<?php echo wp_create_nonce('autobloggpt_nonce'); ?>', // Security nonce
                        license_key: licenseKey // License key from form
                    },
                    success: function(response) {
                        if (response.success) {
                            $('#autobloggpt-message').html('<div class="notice notice-success"><p>' + response.data.message + '</p></div>');
                        } else {
                            $('#autobloggpt-message').html('<div class="notice notice-error"><p>' + response.data.message + '</p></div>');
                        }
                    },
                    error: function() {
                        $('#autobloggpt-message').html('<div class="notice notice-error"><p>An error occurred while processing your request.</p></div>');
                    }
                });
            });

            // Toggle license key visibility
            $('#toggle-password').on('click', function() {
                var $licenseField = $('#autobloggpt_license_key');
                var type = $licenseField.attr('type') === 'password' ? 'text' : 'password';
                $licenseField.attr('type', type);
                $(this).text($(this).text() === 'Show' ? 'Hide' : 'Show');
            });
        });
    </script>

    <script type="text/javascript">
        jQuery(document).ready(function($) {
            // Function to fetch or renew the security code
            function fetchSecurityCode(renew = false) {
                $('#security-code-message').html(renew ? 'Renewing security code...' : 'Requesting security code...');

                // Define the web URL to send as a parameter (using site URL as an example)
                var webUrl = '<?php echo get_site_url(); ?>';

                $.ajax({
                    url: 'https://codemenschen.at/wp-json/autobloggpt/v1/generate-code', // REST API endpoint
                    type: 'POST',
                    contentType: 'application/json', // Set content type to JSON
                    data: JSON.stringify({
                        web_url: webUrl,
                        renew: renew
                    }), // Send the web_url and renew parameter
                    success: function(response) {
                        if (response.code) {
                            // Update the displayed security code with the new or retrieved code from the API
                            $('#current-security-code').text(response.code);
                            $('#security-code-message').html('<div class="notice notice-success"><p>' + response.message + '</p></div>');
                            saveCodeLocally(response.code);
                            location.reload(); // Reload the page to reflect the new code

                        } else {
                            $('#security-code-message').html('<div class="notice notice-error"><p>Failed to update the security code.</p></div>');
                        }
                    },
                    error: function() {
                        $('#security-code-message').html('<div class="notice notice-error"><p>An error occurred while processing the request.</p></div>');
                    }
                });
            }

            // Function to save the fetched code locally as autobloggpt_security_code
            function saveCodeLocally(code) {
                $.ajax({
                    url: '<?php echo get_site_url(); ?>/wp-json/autobloggpt/v1/save-code', // Local endpoint to save the code
                    type: 'POST',
                    contentType: 'application/json',
                    data: JSON.stringify({
                        code: code
                    }),
                    success: function() {
                        console.log("Code saved successfully as autobloggpt_security_code.");
                    },
                    error: function() {
                        console.log("Failed to save the code as autobloggpt_security_code.");
                    }
                });
            }
            var securityCodeExists = <?php echo !empty($security_code) ? 'true' : 'false'; ?>;

            if (!securityCodeExists) {
                fetchSecurityCode(); // Fetch code only if no code exists
            }

            // Handle click event for the "Renew Code" button
            $('#renew-security-code').on('click', function() {
                fetchSecurityCode(true); // Pass true to indicate code renewal
            });
        });
    </script>


    <script>
        function loadContent(tabId) {
            // Hide all tab content
            document.querySelectorAll('.abgpt-tab-content').forEach(function(tab) {
                tab.style.display = 'none';
            });

            // Show the clicked tab content
            var content = document.getElementById(tabId);
            if (content) {
                content.style.display = 'block';
            }

            // Handle active class for sidebar links
            document.querySelectorAll('.abgpt-tab-link').forEach(function(tabLink) {
                tabLink.classList.remove('abgpt-nav-tab-active');
            });

            document.querySelector('a[href="#' + tabId + '"]').classList.add('abgpt-nav-tab-active');
        }
    </script>
<?php
}

// Handle the AJAX request in WordPress
add_action('wp_ajax_activate_autobloggpt_license', 'activate_autobloggpt_license_ajax');

function activate_autobloggpt_license_ajax()
{
    // Check nonce for security
    check_ajax_referer('autobloggpt_nonce', 'security');

    // Get the license key from the AJAX request
    $license_key = sanitize_text_field($_POST['license_key']);

    // Call the license activation function
    $is_activated = activate_autobloggpt_license_key($license_key);

    if ($is_activated) {
        // License activated successfully
        update_option('autobloggpt_license_key', $license_key);
        wp_send_json_success(array('message' => 'License activated successfully!'));
    } else {
        // License activation failed
        wp_send_json_error(array('message' => 'License activation failed. Please check your license key.'));
    }

    wp_die(); // End AJAX execution
}



function activate_autobloggpt_license_key($license_key)
{
    // Your EDD store URL
    $api_url = 'https://codemenschen.at'; // Replace with your EDD store URL

    // API parameters for license activation
    $api_params = array(
        'edd_action' => 'activate_license',  // EDD action for license activation
        'license' => $license_key,        // The license key entered by the user
        'item_name' => urlencode('Autobloggpt'), // Replace with your EDD product name
        'url' => home_url(),          // Current site URL
    );

    // Make the request to the EDD API
    $response = wp_remote_post(add_query_arg($api_params, $api_url), array('timeout' => 15, 'sslverify' => false));
    // Check if there was an error with the request
    if (is_wp_error($response)) {
        return false; // Return false if there was an error
    }

    // Decode the API response
    $license_data = json_decode(wp_remote_retrieve_body($response));

    // Check if the license was activated successfully
    if ($license_data->license === 'valid') {
        return true; // License is valid and activated
    } else {
        return false; // License activation failed
    }
}


// Callback function to save the code as autobloggpt_security_code
function save_autobloggpt_security_code(WP_REST_Request $request)
{
    $code = sanitize_text_field($request->get_param('code'));

    if (empty($code)) {
        return new WP_Error('missing_code', 'The code parameter is required.', array('status' => 400));
    }

    // Save the code as autobloggpt_security_code
    update_option('autobloggpt_security_code', $code);

    return new WP_REST_Response(array(
        'message' => 'Security code saved successfully'
    ), 200);
}
