<?php



function get_post_data_by_url(WP_REST_Request $request)
{
    // Parse the URL to extract the path
    $url = sanitize_text_field($request->get_param('url'));
    $parsed_url = parse_url($url);
    $path = trim($parsed_url['path'], '/'); // Remove leading and trailing slashes

    // Get the post object based on the path, check for both page and post types
    $post = get_page_by_path($path, OBJECT, ['page', 'post']); // Specifically checking for 'page' and 'post'

    if (!$post) {
        return new WP_Error('not_found', 'No page or post found for this URL', array('status' => 404));
    }

    // Prepare the response data
    $data = [
        'id' => $post->ID,
        'title' => get_the_title($post),
        'content' => apply_filters('the_content', $post->post_content),
        'excerpt' => get_the_excerpt($post),
        'status' => $post->post_status,
        'type' => $post->post_type, // To distinguish between 'page' and 'post'
        'url' => get_permalink($post),
        'date' => $post->post_date,
        'modified' => $post->post_modified,
    ];
    ob_clean();
    return new WP_REST_Response($data, 200);
}




function fetch_wp_site_data(WP_REST_Request $request)
{
    $data = [];

    // Get all registered menus
    $menus = wp_get_nav_menus();

    // Initialize homepage_menu to null
    $data['homepage_menu'] = null;

    // Loop through each menu to check for items linking to the homepage
    foreach ($menus as $menu) {
        $menu_items = wp_get_nav_menu_items($menu->term_id);

        // Initialize a flag to check if the homepage is found in the current menu
        $homepage_found = false;

        foreach ($menu_items as $item) {
            // Check if the item URL matches the homepage URL
            if (home_url('/') === $item->url) {
                // If it matches, set homepage_menu and break the loop
                $data['homepage_menu'] = [
                    'name' => $menu->name,
                    'description' => $menu->description,
                    'items' => [], // To store menu items
                ];

                // Store menu items in the response
                foreach ($menu_items as $menu_item) {
                    $data['homepage_menu']['items'][] = [
                        'id' => $menu_item->ID,
                        'title' => $menu_item->title,
                        'url' => $menu_item->url,
                    ];
                }

                // Break out of the loop since we found the homepage menu
                $homepage_found = true;
                break; // Exit the inner loop
            }
        }

        // Break the outer loop if homepage menu has been found
        if ($homepage_found) {
            break;
        }
    }

    // Check if homepage_menu is still null and handle accordingly
    if (is_null($data['homepage_menu'])) {
        $data['homepage_menu'] = 'No menu found that links to the homepage.';
    }

    $data['list_menu'] = $menus;

    // Fetch additional data as needed
    $data['list_page'] = get_pages();
    $data['list_post'] = get_posts();
    ob_clean();
    return new WP_REST_Response($data, 200);
}



function autobloggpt_get_post(WP_REST_Request $request)
{
    $post_id = intval($request->get_param('post_id'));

    if (empty($post_id) || !get_post($post_id)) {
        return new WP_Error('invalid_post_id', 'Invalid post ID', array('status' => 400));
    }

    $post = get_post($post_id);
    $post_data = array(
        'post_id' => $post->ID,
        'title' => get_the_title($post->ID),
        'content' => apply_filters('the_content', $post->post_content),
        'date' => get_the_date('', $post->ID),
        'author' => get_the_author_meta('display_name', $post->post_author),
        'categories' => wp_get_post_categories($post->ID, array('fields' => 'names')),
        'tags' => wp_get_post_tags($post->ID, array('fields' => 'names')),
        'image_url' => get_the_post_thumbnail_url($post->ID),
    );
    ob_clean();
    return new WP_REST_Response($post_data, 200);
}


// Function to handle storing custom preview data
function autobloggpt_preview_post(WP_REST_Request $request)
{
    // Extract post ID from the request
    $post_id = intval($request->get_param('post_id'));

    // Check if the post exists
    $post = get_post($post_id);
    if (!$post) {
        return new WP_Error('invalid_post_id', 'Invalid post ID', array('status' => 400));
    }

    // Extract custom title, content, and tags from the request
    $custom_title = sanitize_text_field($request->get_param('title'));
    $custom_content = wp_kses_post($request->get_param('content'));
    // Extract and sanitize tags array
    $custom_tags = $request->get_param('tags');
    if (is_array($custom_tags)) {
        $custom_tags = array_map('sanitize_text_field', $custom_tags); // Sanitize each tag
    } else {
        $custom_tags = []; // If it's not an array, fallback to empty
    }

    // Create a unique nonce for the preview session
    $preview_nonce = wp_create_nonce('autobloggpt_preview_' . time());

    // Store the custom data in a transient for 1 hour
    $preview_data = array(
        'title'   => $custom_title,
        'content' => $custom_content,
        'tags'    => $custom_tags,
    );
    set_transient('autobloggpt_preview_' . $preview_nonce, $preview_data, 3600); // Expires in 1 hour

    // Generate the native WordPress preview URL
    $preview_url = get_preview_post_link($post_id);

    // Append the preview_nonce to the preview URL
    $preview_url = add_query_arg(array('preview_nonce' => $preview_nonce), $preview_url);

    $response = wp_remote_get($preview_url);

    $html_source = wp_remote_retrieve_body($response);
    ob_clean();
    // Return the preview URL
    return new WP_REST_Response(array('preview_url' => $preview_url, 'html_source' => $html_source), 200);
}

/**
 * Fetch the HTML source of a given URL.
 *
 * @param string $url The URL to fetch.
 * @return WP_REST_Response|WP_Error The HTML source or an error response.
 */
function fetch_html_source(WP_REST_Request $request)
{

    $url = sanitize_text_field($request->get_param('url'));
    // Validate the URL
    if (!filter_var($url, FILTER_VALIDATE_URL)) {
        return new WP_Error('invalid_url', 'The provided URL is invalid.', array('status' => 400));
    }

    // Use wp_remote_get to fetch the URL content
    $response = wp_remote_get($url);
    // Check for errors in the response
    if (is_wp_error($response)) {
        return new WP_Error('fetch_error', 'Error fetching the URL: ' . $response->get_error_message(), array('status' => 500));
    }

    // Get the response code
    $response_code = wp_remote_retrieve_response_code($response);

    // Check if the response code is OK (200)
    if ($response_code !== 200) {
        return new WP_Error('http_error', 'Failed to fetch content. HTTP Response Code: ' . $response_code, array('status' => $response_code));
    }

    // Retrieve the body of the response
    $html_source = wp_remote_retrieve_body($response);
    ob_clean();
    // Return the HTML source as a REST response
    return new WP_REST_Response(array('html_source' => $html_source), 200);
}


// Function to modify post data during preview
// Modify the post during preview
function autobloggpt_modify_preview_post($post)
{
    // Only modify the post data if it's a preview and if a preview_nonce is passed in the URL
    if (is_preview() && isset($_GET['preview_nonce'])) {
        $preview_nonce = sanitize_text_field($_GET['preview_nonce']);

        // Retrieve the stored custom data from the transient
        $preview_data = get_transient('autobloggpt_preview_' . $preview_nonce);

        if ($preview_data) {
            // Override the post title and content with the stored custom data
            if (!empty($preview_data['title'])) {
                $post->post_title = $preview_data['title'];
            }
            if (!empty($preview_data['content'])) {
                $post->post_content = $preview_data['content'];
            }

            // Handle tags - wp_set_post_tags doesn't work for preview, so handle this separately if needed
        }
    }

    return $post;
}
add_filter('the_post', 'autobloggpt_modify_preview_post');


